#include <CrashTick.h>

#include <crashtick/misc.h>

#include <clib/alib_protos.h>
#include <clib/dos_protos.h>

uchar announcebuf[1000];

extern struct Config config;

struct AnnLine
{
   struct AnnLine *Next;
   struct Area *Area;
   uchar *Line;
};

extern APTR mempool;

uchar *table[30];

void MakeAnnounceLine(uchar *dest,ulong destsize,uchar *fmt,uchar **table)
{
   ulong c,d,e;
   ulong width,trunc;
   short left,haspercent,settrunc;

   d=0;

   haspercent=FALSE;

   for(c=0;c<strlen(fmt) && d!=destsize-1;c++)
   {
      if(haspercent)
      {
         if(fmt[c] >= '0' && fmt[c] <= '9')
         {
            if(settrunc)
               trunc=trunc*10+fmt[c]-'0';

            else
               width=width*10+fmt[c]-'0';
         }
         else if(fmt[c]=='.')
         {
            settrunc=TRUE;
         }
         else if(fmt[c]=='-')
         {
            left=TRUE;
         }
         else if(fmt[c]>='a' && fmt[c]<='z' && table[fmt[c]-'a'])
         {
            if(!left && width!=0 && strlen(table[fmt[c]-'a'])<width)
            {
               for(e=0;e<width-strlen(table[fmt[c]-'a']) && d!=destsize-1;e++)
                  dest[d++]=' ';
            }

            if(trunc!= 0 && trunc < destsize-1-d)
               strncpy(&dest[d],table[fmt[c]-'a'],trunc);

            else
               strncpy(&dest[d],table[fmt[c]-'a'],destsize-1-d);

            dest[destsize-1]=0;
            d=strlen(dest);

            if(left && width!=0 && strlen(table[fmt[c]-'a'])<width)
            {
               for(e=0;e<width-strlen(table[fmt[c]-'a']) && d!=destsize-1;e++)
                  dest[d++]=' ';
            }

            haspercent=FALSE;
         }
         else
         {
            dest[d++]=fmt[c];
            haspercent=FALSE;
         }
      }
      else
      {
         if(fmt[c]=='\\' && (fmt[c+1]|32)=='n')
         {
            strncpy(&dest[d],"\n",destsize-1-d);
            dest[destsize-1]=0;
            d=strlen(dest);
            c++;
         }
         else if(fmt[c]=='%')
         {
            settrunc=FALSE;
            left=FALSE;
            width=0;
            trunc=0;

            haspercent=TRUE;
         }
         else
         {
            dest[d++]=fmt[c];
         }
      }
   }

   dest[d]=0;
}


short AddAnnounce(struct Area *area,
   uchar *areaname,
   uchar *filename,
   uchar *path,
   uchar *fullfile,
   uchar *desc,
   uchar *fromnode,
   uchar *magic,
   uchar *origin,
   uchar *size)
{
   struct Announce *announce;
   struct AnnLine *annline;
   short c;

   for(announce=(struct Announce *)config.AnnounceList.First;announce;announce=announce->Next)
   {
      if(CheckFlags(area->Group,announce->Groups))
      {
         for(c=0;c<30;c++)
            table[c]=NULL;

         table['a'-'a']=areaname;
         table['n'-'a']=filename;
         table['p'-'a']=path;
         table['s'-'a']=fullfile;
         table['b'-'a']=size;
         table['o'-'a']=origin;
         table['f'-'a']=fromnode;
         table['m'-'a']=magic;
         table['d'-'a']=desc;

         MakeAnnounceLine(announcebuf,1000,announce->Format,table);

         if(!(annline=AllocCleared(sizeof(struct AnnLine))))
         {
            return(FALSE);
         }

         if(!(annline->Line=AllocCleared(strlen(announcebuf)+1)))
         {
            LibFreePooled(mempool,annline,sizeof(struct AnnLine));
            return(FALSE);
         }

         annline->Area=area;
         strcpy(annline->Line,announcebuf);
         jbAddNode(&announce->LineList,(struct jbNode *)annline);
      }
   }
   return(TRUE);
}

void FreeAnnounces(void)
{
   struct Announce *announce;
   struct AnnLine *annline;

   for(announce=(struct Announce *)config.AnnounceList.First;announce;announce=announce->Next)
   {
      for(annline=(struct AnnLine *)announce->LineList.First;annline;annline=annline->Next)
         LibFreePooled(mempool,annline->Line,strlen(annline->Line)+1);

      jbFreeList(&announce->LineList,sizeof(struct AnnLine));
   }
}

void FileToPacket(uchar *filename,struct jbFile *jb)
{
   BPTR fh;
   ulong rlen,c;

   if(!(fh=Open(filename,MODE_OLDFILE)))
   {
      LogWrite(4,RAID,getString(ERR_FAILED_OPEN_READ),filename);
      return;
   }

   while(rlen=Read(fh,announcebuf,500))
   {
      for(c=0;c<rlen;c++)
         if(announcebuf[c]==10) announcebuf[c]=13;

      jbWrite(jb,announcebuf,rlen);
   }

   Close(fh);
}

void FileToFile(uchar *filename,struct jbFile *jb)
{
   BPTR fh;
   ulong rlen;

   if(!(fh=Open(filename,MODE_OLDFILE)))
   {
      LogWrite(4,RAID,getString(ERR_FAILED_OPEN_READ),filename);
      return;
   }

   while(rlen=Read(fh,announcebuf,500))
      jbWrite(jb,announcebuf,rlen);

   Close(fh);
}

void jbWriteCR(struct jbFile *jb,uchar *announcebuf)
{
   ulong c;

   for(c=0;announcebuf[c]!=0;c++)
      if(announcebuf[c] == 10) announcebuf[c]=13;

   jbFPrintf(jb,"%s",announcebuf);
}

int CompareAnnLines(struct AnnLine **l1,struct AnnLine **l2)
{
   if((*l1)->Area == (*l2)->Area)
      return stricmp((*l1)->Line,(*l2)->Line);

   return stricmp((*l1)->Area->Name,(*l2)->Area->Name);
}

void SortAnnLines(struct jbList *list)
{
   ULONG nc,alloc;
   struct AnnLine *al,**buf,**work;

   nc=0;

   for(al=(struct AnnLine *)list->First;al;al=al->Next)
      nc++;

   if(nc==0)
      return;

   alloc=nc*4;

   if(!(buf=(struct AreaStatsNode **)LibAllocPooled(mempool,alloc)))
      return;

   work=buf;

   for(al=(struct AnnLine *)list->First;al;al=al->Next)
      *work++=al;

   qsort(buf,nc,4,CompareAnnLines);
   jbNewList(list);

   for(work=buf;nc--;)
      jbAddNode(list,(struct jbNode *)*work++);

   LibFreePooled(mempool,buf,alloc);
}

void Announce(void)
{
   struct Announce *announce;
   struct AnnLine *annline;
   struct jbFile *jb;
   struct Area *lastarea;
   uchar addr[30];
   uchar errorbuf[100];
   ulong c;

   jb=NULL;

   for(announce=(struct Announce *)config.AnnounceList.First;announce;announce=announce->Next)
   {
      if(announce->LineList.First && announce->Method == ANNOUNCE_PKT)
      {
         SortAnnLines(&announce->LineList);

         if(!jb)
         {
            if(!(jb=CreatePacket(&announce->Aka->Node,errorbuf)))
            {
               LogWrite(1,TICKERR,"%s",errorbuf);
               FreeAnnounces();
               return;
            }
         }

         WriteEchoHeader(jb,config.cfg_Sysop,&announce->Aka->Node,"All",announce->Subject,announce->Name);

         if(announce->Header[0])
            FileToPacket(announce->Header,jb);

         lastarea=NULL;

         for(annline=(struct AnnLine *)announce->LineList.First;annline;annline=annline->Next)
         {
            for(c=0;c<30;c++)
               table[c]=NULL;

            table['a'-'a']=annline->Area->Name;
            table['d'-'a']=annline->Area->Description;

            if(lastarea != annline->Area && announce->AreaHeader[0])
            {
               MakeAnnounceLine(announcebuf,1000,announce->AreaHeader,table);
               jbWriteCR(jb,announcebuf);
            }

            strcpy(announcebuf,annline->Line);
            jbWriteCR(jb,announcebuf);

            if(lastarea != annline->Area && announce->AreaFooter[0])
            {
               MakeAnnounceLine(announcebuf,1000,announce->AreaFooter,table);
               jbWriteCR(jb,announcebuf);
            }

            lastarea=annline->Area;
         }

         if(announce->Footer[0])
            FileToPacket(announce->Footer,jb);

         if(announce->Aka->Node.Point)
            sprintf(addr,"%lu:%lu/%lu.%lu",
               announce->Aka->Node.Zone,
               announce->Aka->Node.Net,
               announce->Aka->Node.Node,
               announce->Aka->Node.Point);

         else
            sprintf(addr,"%lu:%lu/%lu",
               announce->Aka->Node.Zone,
               announce->Aka->Node.Net,
               announce->Aka->Node.Node);

         jbFPrintf(jb,"---\x0d");
         jbFPrintf(jb," * Origin: %s (%s)\x0d",announce->Origin,addr);
         jbPutChar(jb,0);
      }
   }

   if(jb) ClosePacket(jb);

   for(announce=(struct Announce *)config.AnnounceList.First;announce;announce=announce->Next)
   {
      if(announce->LineList.First && announce->Method == ANNOUNCE_FILE)
      {
         SortAnnLines(&announce->LineList);

         if(!(jb=jbOpen(announce->Name,MODE_READWRITE,5000)))
         {
            LogWrite(1,TICKERR,getString(ERR_FAILED_OPEN_READ),announce->Name);
            FreeAnnounces();
            return;
         }

         Seek(jb->fh,0,OFFSET_END); /* Go to end of file */

         if(announce->Header[0])
            FileToFile(announce->Header,jb);

         lastarea=NULL;

         for(annline=(struct AnnLine *)announce->LineList.First;annline;annline=annline->Next)
         {
            for(c=0;c<30;c++)
               table[c]=NULL;

            table['a'-'a']=annline->Area->Name;
            table['d'-'a']=annline->Area->Description;

            if(lastarea != annline->Area && announce->AreaHeader[0])
            {
               MakeAnnounceLine(announcebuf,1000,announce->AreaHeader,table);
               jbWrite(jb,announcebuf,strlen(announcebuf));
            }

            strcpy(announcebuf,annline->Line);
            jbWrite(jb,announcebuf,strlen(announcebuf));

            if(lastarea != annline->Area && announce->AreaFooter[0])
            {
               MakeAnnounceLine(announcebuf,1000,announce->AreaFooter,table);
               jbWrite(jb,announcebuf,strlen(announcebuf));
            }

            lastarea=annline->Area;
         }

         if(announce->Footer[0])
            FileToFile(announce->Footer,jb);

         jbClose(jb);
      }
   }

   FreeAnnounces();
}
