#include <CrashTick.h>
#include <shared/fidonet.h>

#include <dos/dos.h>
#include <dos/datetime.h>
#include <clib/dos_protos.h>

extern struct Config config;

struct PktHeader PktHeader;
struct PktMsgHeader PktMsgHeader;

struct jbFile *CreatePacket(struct Node4D *node,uchar *cfgerr)
{
   UBYTE buf[100],buf2[100];
   struct jbFile *jb;
   struct DateTime dt;
   struct DateStamp ds;
   UBYTE date[LEN_DATSTRING];
   ULONG num;
   BPTR l;

   DateStamp(&ds);
   num=ds.ds_Days*24*60*60+ds.ds_Minute*60+ds.ds_Tick/50;

   jb=FALSE;

   while(!jb)
   {
      sprintf(buf2,"%08lx.pkt",num);
      strcpy(buf,config.cfg_Inbound);
      AddPart(buf,buf2,100);

      l=Lock(buf,SHARED_LOCK);

      while(l || IoErr()==ERROR_OBJECT_IN_USE)
      {
         if(l) UnLock(l);
         num++;

         sprintf(buf2,"%08lx.pkt",num);
         strcpy(buf,config.cfg_Inbound);
         AddPart(buf,buf2,100);

         l=Lock(buf,SHARED_LOCK);
      }

      if(!(jb=jbOpen(buf,MODE_NEWFILE,5000)))
      {
         if(IoErr()==ERROR_OBJECT_IN_USE)
            num++;

         else
         {
            sprintf(cfgerr,getString(ERR_FAILED_OPEN_WRITE),buf);
            return(NULL);
         }
      }
   }

   PktHeader.OrigNode=SWAP(config.cfg_Fake4D.Node);
   PktHeader.DestNode=SWAP(node->Node);

   DateStamp(&dt.dat_Stamp);
   dt.dat_Format=FORMAT_USA;
   dt.dat_Flags=0;
   dt.dat_StrDay=NULL;
   dt.dat_StrDate=date;
   dt.dat_StrTime=NULL;
   DateToStr(&dt);

   PktHeader.Month=atoi(date)-1;
   PktHeader.Day=atoi(&date[3]);
   PktHeader.Month=SWAP(PktHeader.Month);
   PktHeader.Day=SWAP(PktHeader.Day);
   PktHeader.Hour=SWAP(dt.dat_Stamp.ds_Minute/60);
   PktHeader.Minute=SWAP(dt.dat_Stamp.ds_Minute%60);
   PktHeader.Second=SWAP(dt.dat_Stamp.ds_Tick/50);

   if(atoi(&date[6])>90)
      PktHeader.Year=atoi(&date[6])+1900;

   else
      PktHeader.Year=atoi(&date[6])+2000;

   PktHeader.Year=SWAP(PktHeader.Year);

   PktHeader.Baud=0;
   PktHeader.PktType=0x0200;
   PktHeader.OrigNet=SWAP(config.cfg_Fake4D.Net);
   PktHeader.DestNet=SWAP(node->Net);
   PktHeader.ProdCodeLow=0xfe;
   PktHeader.RevisionMajor=VERSION_MAJOR;
   PktHeader.QOrigZone=SWAP(config.cfg_Fake4D.Zone);
   PktHeader.QDestZone=SWAP(node->Zone);
   PktHeader.AuxNet=0;
   PktHeader.CWValidCopy=1;
   PktHeader.ProdCodeHigh=0;
   PktHeader.RevisionMinor=VERSION_MINOR;
   PktHeader.CapabilWord=SWAP(1);
   PktHeader.OrigZone=SWAP(config.cfg_Fake4D.Zone);
   PktHeader.DestZone=SWAP(node->Zone);
   PktHeader.OrigPoint=SWAP(config.cfg_Fake4D.Point);
   PktHeader.DestPoint=SWAP(node->Point);
   PktHeader.ProdData=0;

   strncpy(PktHeader.Password,config.cfg_FakePW,8);

   jbWrite(jb,&PktHeader,sizeof(struct PktHeader));
   return(jb);
}

void ClosePacket(struct jbFile *jb)
{
   uchar filename[200];

   jbPutChar(jb,0);
   jbPutChar(jb,0);

   NameFromFH(jb->fh,filename,200);
   jbClose(jb);
   SetComment(filename,"No security");
}

void WriteNull(struct jbFile *jb,UBYTE *str)
{
   jbWrite(jb,str,strlen(str)+1);
}

UBYTE *MonthNames[13]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec","???"};

void WriteHeader(struct jbFile *jb,uchar *from,uchar *to,struct Node4D *from4d,struct Node4D *to4d,uchar *subj)
{
   uchar datetime[20];
   struct DateTime dt;
   UBYTE date[40],time[40];

   PktMsgHeader.PktType=0x0200;
   PktMsgHeader.OrigNode=SWAP(from4d->Node);
   PktMsgHeader.DestNode=SWAP(to4d->Node);
   PktMsgHeader.OrigNet=SWAP(from4d->Net);
   PktMsgHeader.DestNet=SWAP(to4d->Net);
   PktMsgHeader.Attr=SWAP(PVT);
   PktMsgHeader.Cost=0;

   jbWrite(jb,&PktMsgHeader,sizeof(struct PktMsgHeader));

   DateStamp(&dt.dat_Stamp);
   dt.dat_Format=FORMAT_CDN;
   dt.dat_Flags=0;
   dt.dat_StrDay=NULL;
   dt.dat_StrDate=date;
   dt.dat_StrTime=time;
   DateToStr(&dt);

   date[2]=0;
   date[5]=0;

   sprintf(datetime,"%s %s %s  %s",date,MonthNames[atoi(&date[3])-1],&date[6],time);

   WriteNull(jb,datetime);
   WriteNull(jb,to);
   WriteNull(jb,from);
   WriteNull(jb,subj);

   if(from4d->Point)
      jbFPrintf(jb,"\x01""FMPT %lu\x0d",from4d->Point);

   if(to4d->Point)
      jbFPrintf(jb,"\x01""TOPT %lu\x0d",to4d->Point);

   jbFPrintf(jb,"\x01""INTL %lu:%lu/%lu %lu:%lu/%lu\x0d",
         to4d->Zone,to4d->Net,to4d->Node,
         from4d->Zone,from4d->Net,from4d->Node);

   jbFPrintf(jb,"\x01""PID: CrashTick "TID_VERSION"\x0d");
}

void WriteEchoHeader(struct jbFile *jb,uchar *from,struct Node4D *from4d,uchar *to,uchar *subj,uchar *area)
{
   uchar datetime[20];
   struct DateTime dt;
   UBYTE date[40],time[40];

   PktMsgHeader.PktType=0x0200;
   PktMsgHeader.OrigNode=SWAP(from4d->Node);
   PktMsgHeader.DestNode=SWAP(from4d->Node);
   PktMsgHeader.OrigNet=SWAP(from4d->Net);
   PktMsgHeader.DestNet=SWAP(from4d->Net);
   PktMsgHeader.Attr=0;
   PktMsgHeader.Cost=0;

   jbWrite(jb,&PktMsgHeader,sizeof(struct PktMsgHeader));

   DateStamp(&dt.dat_Stamp);
   dt.dat_Format=FORMAT_CDN;
   dt.dat_Flags=0;
   dt.dat_StrDay=NULL;
   dt.dat_StrDate=date;
   dt.dat_StrTime=time;
   DateToStr(&dt);

   date[2]=0;
   date[5]=0;

   sprintf(datetime,"%s %s %s  %s",date,MonthNames[atoi(&date[3])-1],&date[6],time);

   WriteNull(jb,datetime);
   WriteNull(jb,to);
   WriteNull(jb,from);
   WriteNull(jb,subj);

   jbFPrintf(jb,"AREA: %s\x0d",area);
   jbFPrintf(jb,"\x01""PID: CrashTick "TID_VERSION"\x0d");
}



