#include <CrashTick.h>
#include <crc.h>
#include <misc.h>

extern APTR mempool;

#include <clib/alib_protos.h>
#include <clib/exec_protos.h>
#include <clib/dos_protos.h>

extern struct Config config;
ulong ticknum;

short ProcessDirectory(uchar *dir);
short ProcessTick(uchar *ticfilename);

void BadTick(uchar *tickname, uchar *filename,uchar *reason);

short Tick(void)
{
   uchar errorbuf[100];
   uchar cmdbuf[200];
   uchar namebuf[200];
   struct FileEntry *fe;
   struct jbList FileList;
   BPTR l,olddir;
   short arcres;

   PrepareTickHatch();

   ticknum=0;

   if(!ProcessDirectory(config.cfg_Inbound))
      return(10);

   if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
      return(0);

   if(!jbReadDir(config.cfg_Inbound,&FileList,IsZick))
   {
      LogWrite(1,TICKERR,getString(ERR_NO_DIRECTORY),config.cfg_Inbound);
      return(FALSE);
   }

   if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
   {
      jbFreeList(&FileList,sizeof(struct FileEntry));
      return(0);
   }

   if(!(l=Lock(config.cfg_Inbound,SHARED_LOCK)))
   {
      jbFreeList(&FileList,sizeof(struct FileEntry));
      UnLock(l);
      return(10);
   }

   for(fe=(struct FileEntry *)FileList.First;fe;fe=fe->Next)
   {
      LogWrite(2,TICKINFO,getString(TXT_UNPACKING_WITH),fe->Name,"ZIP");
      ExpandPacker(cmdbuf,200,config.cfg_ZipExtract,fe->Name,"");
      LogWrite(6,DEBUG,getString(TXT_EXECUTING_COMMAND),cmdbuf);
      olddir=CurrentDir(l);
      arcres=SystemTags(cmdbuf,TAG_DONE);
      CurrentDir(olddir);

      if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         UnLock(l);
         return(0);
      }

      strcpy(namebuf,config.cfg_Inbound);
      AddPart(namebuf,fe->Name,200);

      if(arcres == 0)
      {
         DeleteFile(namebuf);
      }
      else
      {
         sprintf(errorbuf,getString(ERR_UNPACKER_FAILED),arcres);
         LogWrite(1,TICKERR,errorbuf);
         BadTick(namebuf,"",errorbuf);
      }

      if(!ProcessDirectory(config.cfg_Inbound))
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         UnLock(l);
         return(10);
      }

      if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         UnLock(l);
         return(0);
      }
   }

   jbFreeList(&FileList,sizeof(struct FileEntry));

   if(!jbReadDir(config.cfg_Inbound,&FileList,IsLick))
   {
      LogWrite(1,TICKERR,getString(ERR_NO_DIRECTORY),config.cfg_Inbound);
      UnLock(l);
      return(10);
   }

   for(fe=(struct FileEntry *)FileList.First;fe;fe=fe->Next)
   {
      LogWrite(2,TICKINFO,getString(TXT_UNPACKING_WITH),fe->Name,"LHA");
      ExpandPacker(cmdbuf,200,config.cfg_LhaExtract,fe->Name,"");
      LogWrite(6,DEBUG,getString(TXT_EXECUTING_COMMAND),cmdbuf);
      olddir=CurrentDir(l);
      arcres=SystemTags(cmdbuf,TAG_DONE);
      CurrentDir(olddir);

      if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         UnLock(l);
         return(0);
      }

      strcpy(namebuf,config.cfg_Inbound);
      AddPart(namebuf,fe->Name,200);

      if(arcres == 0)
      {
         DeleteFile(namebuf);
      }
      else
      {
         sprintf(errorbuf,getString(ERR_UNPACKER_FAILED),arcres);
         LogWrite(1,TICKERR,errorbuf);
         BadTick(namebuf,"",errorbuf);
      }

      if(!ProcessDirectory(config.cfg_Inbound))
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         UnLock(l);
         return(10);
      }

      if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         UnLock(l);
         return(0);
      }
   }

   jbFreeList(&FileList,sizeof(struct FileEntry));
   UnLock(l);

   if(ticknum == 0)
   {
      LogWrite(4,TICKINFO,getString(TXT_NOTHING_TO_PROCESS));
      return(0);
   }

   if(config.configchanged)
   {
      if(!WriteConfig(&config,errorbuf))
         LogWrite(1,SYSTEMERR,errorbuf);
   }

   Announce();
   ProcessWriteStats();

   return(0);
}

short ProcessDirectory(uchar *dir)
{
   struct jbList FileList;
   struct FileEntry *fe;
   uchar tickname[200];

   if(!jbReadDir(dir,&FileList,IsTick))
   {
      LogWrite(1,TICKERR,getString(ERR_NO_DIRECTORY),dir);
      return(FALSE);
   }

   for(fe=(struct FileEntry *)FileList.First;fe;fe=fe->Next)
   {
      if(SetSignal(0L,0L) & SIGBREAKF_CTRL_C)
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         return(TRUE);
      }

      strcpy(tickname,dir),
      AddPart(tickname,fe->Name,200);

      if(!ProcessTick(tickname))
      {
         jbFreeList(&FileList,sizeof(struct FileEntry));
         return(FALSE);
      }

      ticknum++;
   }

   jbFreeList(&FileList,sizeof(struct FileEntry));
   return(TRUE);
}

void BadTick(uchar *tickname, uchar *filename,uchar *reason)
{
   uchar buf[200],commentbuf[100];
   uchar faultbuf[100];

   if(config.cfg_BadTick[0]==0)
      return;

   strcpy(buf,config.cfg_BadTick);
   AddPart(buf,FilePart(tickname),200);

   LogWrite(5,TICKINFO,getString(TXT_MOVING_FILE),tickname,buf);

   if(!MoveFile(tickname,buf))
   {
      LogWrite(1,TICKERR,getString(ERR_FAILED_TO_MOVE),tickname,buf);
      Fault(IoErr(),NULL,faultbuf,100);
      LogWrite(1,TICKERR,getString(ERR_IO_ERROR),faultbuf);
   }
   else
   {
      SetComment(buf,reason);
   }

   if(filename[0]==0)
      return;

   strcpy(buf,config.cfg_BadTick);
   AddPart(buf,FilePart(filename),200);

   LogWrite(5,TICKINFO,getString(TXT_MOVING_FILE),filename,buf);

   if(!MoveFile(filename,buf))
   {
      LogWrite(1,TICKERR,getString(ERR_FAILED_TO_MOVE),filename,buf);
      Fault(IoErr(),NULL,faultbuf,100);
      LogWrite(1,TICKERR,getString(ERR_IO_ERROR),faultbuf);
   }
   else
   {
      sprintf(commentbuf,getString(ERR_BELONGS_TO),FilePart(tickname));
      SetComment(buf,commentbuf);
   }
}

void MakeDirectory(UBYTE *dest,ULONG destsize,UBYTE *defdir,UBYTE *areaname)
{
   ULONG c,d;

   d=0;
   for(c=0;c<strlen(defdir) && d!=destsize-1;c++)
   {
      if(defdir[c]=='%' && (defdir[c+1]|32)=='a')
      {
         strncpy(&dest[d],areaname,destsize-1-d);
         dest[destsize-1]=0;
         d=strlen(dest);
         c++;
      }
      else dest[d++]=defdir[c];
   }
   dest[d]=0;
}

short ProcessTick(uchar *ticfilename)
{
   uchar fullfilename[200];
   uchar filename[100],areaname[100],from[100],pw[100];
   ulong crc,c;
   struct Area *area;
   struct jbList tick;
   struct jbFile *jb;
   uchar linebuf[200];
   uchar errorbuf[100];
   struct TickLine *tickline;
   struct Node4D n4d;
   struct ConfigNode *cnode;
   struct TickNode *tn;
   short res;
   BPTR l;
   
   jbNewList(&tick);

   printf("\n");
   LogWrite(1,TICKINFO,getString(TXT_TICK_PROCESSING),FilePart(ticfilename));

   if(!(jb=jbOpen(ticfilename,MODE_OLDFILE,2000)))
   {
      LogWrite(1,TICKERR,getString(ERR_FAILED_OPEN_READ),ticfilename);
      return(TRUE);
   }

   while(jbFGets(jb,linebuf,199))
   {
      if(linebuf[0] && linebuf[strlen(linebuf)-1] == 13)
         linebuf[strlen(linebuf)-1]=0;

      if(linebuf[0] && linebuf[strlen(linebuf)-1] == 10)
         linebuf[strlen(linebuf)-1]=0;

      if(!(tickline=AllocCleared(sizeof(struct TickLine))))
      {
         LogWrite(1,SYSTEMERR,getString(ERR_OUT_OF_MEMORY));
         jbFreeList(&tick,sizeof(struct TickLine));
         return(FALSE);
      }

      strcpy(tickline->line,linebuf);
      jbAddNode(&tick,(struct jbNode *)tickline);
   }

   jbClose(jb);

   filename[0]=0;
   areaname[0]=0;
   from[0]=0;
   pw[0]=0;

   crc=0xffffffff;

   for(tickline=(struct TickLine *)tick.First;tickline;tickline=tickline->Next)
   {
      jbcpos=0;

      if(jbstrcpy(linebuf,tickline->line,50))
      {
         if(stricmp(linebuf,"File")==0)
            if(!jbstrcpy(filename,tickline->line,100)) filename[0]=0;

         if(stricmp(linebuf,"Area")==0)
            if(!jbstrcpy(areaname,tickline->line,100)) areaname[0]=0;

         if(stricmp(linebuf,"From")==0)
            if(!jbstrcpy(from,tickline->line,100)) from[0]=0;

         if(stricmp(linebuf,"Pw")==0)
            if(!jbstrcpy(pw,tickline->line,100)) pw[0]=0;

         if(stricmp(linebuf,"CRC")==0)
         {
            if(jbstrcpy(linebuf,tickline->line,100))
               crc=hextodec(linebuf);
         }
      }
   }

   fullfilename[0]=0;

   if(filename[0])
   {
      strcpy(fullfilename,ticfilename);
      *PathPart(fullfilename)=0;
      AddPart(fullfilename,filename,200);
   }
   
   if(!filename[0] || !areaname[0] || !from[0])
   {
      if(!areaname[0])
         sprintf(errorbuf,getString(ERR_NO_AREA_LINE),ticfilename);           

       if(!filename[0])
         sprintf(errorbuf,getString(ERR_NO_FILE_LINE),ticfilename);

       if(!from[0])
         sprintf(errorbuf,getString(ERR_NO_FROM_LINE),ticfilename);

      LogWrite(1,TICKERR,errorbuf);
      BadTick(ticfilename,fullfilename,errorbuf);

      jbFreeList(&tick,sizeof(struct TickLine));
      return(TRUE);
   }

   /* Remove domain */

   for(c=0;from[c]!=0;c++)
      if(from[c]=='@') from[c]=0;

   if(!Parse4D(from,&n4d))
   {
      sprintf(errorbuf,getString(ERR_INVALID_NODE),from);
      LogWrite(1,TICKERR,errorbuf);
      BadTick(ticfilename,fullfilename,errorbuf);

      jbFreeList(&tick,sizeof(struct TickLine));
      return(TRUE);
   }

   for(cnode=(struct ConfigNode *)config.CNodeList.First;cnode;cnode=cnode->Next)
      if(Compare4D(&n4d,&cnode->Node)==0) break;

   if(!cnode)
   {
      sprintf(errorbuf,getString(ERR_UNKNOWN_NODE), 
         n4d.Zone,
         n4d.Net,
         n4d.Node,
         n4d.Point);

      LogWrite(1,TICKERR,errorbuf);
      BadTick(ticfilename,fullfilename,errorbuf);

      jbFreeList(&tick,sizeof(struct TickLine));
      return(TRUE);
   }

   if(cnode->TickPW[0])
   {
      if(stricmp(cnode->TickPW,pw)!=0)
      {
         sprintf(errorbuf,getString(ERR_WRONG_PASSWORD),
            pw,
            n4d.Zone,
            n4d.Net,
            n4d.Node,
            n4d.Point);

         LogWrite(1,TICKERR,errorbuf);
         BadTick(ticfilename,fullfilename,errorbuf);

         jbFreeList(&tick,sizeof(struct TickLine));
         return(TRUE);
      }
   }

   for(area=(struct Area *)config.AreaList.First;area;area=area->Next)
      if(stricmp(area->Name,areaname)==0) break;

   if(!area)
   {
      area=AllocCleared(sizeof(struct Area));
      tn=AllocCleared(sizeof(struct TickNode));

      if(!area || !tn)
      {
         if(area) LibFreePooled(mempool,area,sizeof(struct Area));
         if(tn)   LibFreePooled(mempool,tn,sizeof(struct TickNode));

         LogWrite(1,SYSTEMERR,getString(ERR_OUT_OF_MEMORY));
         jbFreeList(&tick,sizeof(struct TickLine));
         return(FALSE);
      }

      area->Aka=BestAka(&cnode->Node);
      area->Group=cnode->DefaultGroup;

      mystrncpy(area->Name,areaname,80);
      strcpy(area->Description,"Auto-added");
      area->AreaCommand[0]=0;
      MakeDirectory(area->Directory,80,config.cfg_AutoAddDir,areaname);

      tn->ConfigNode=cnode;

      jbNewList(&area->TickNodes);
      jbAddNode(&area->TickNodes,(struct jbNode *)tn);
      jbAddNode(&config.AreaList,(struct jbNode *)area);
      config.configchanged=TRUE;

      if(cnode->Flags & NODE_AUTOADD)
      {
         if(l=CreateDir(area->Directory))
            UnLock(l);

         LogWrite(1,TICKERR,getString(ERR_UNKNOWN_AREA_AUTOADD),areaname);
      }
      else
      {
         area->Flags |= AREA_UNCONFIRMED;

         sprintf(errorbuf,getString(ERR_UNKNOWN_AREA),areaname);
         LogWrite(1,TICKERR,errorbuf);
         BadTick(ticfilename,fullfilename,errorbuf);

         jbFreeList(&tick,sizeof(struct TickLine));
         return(TRUE);
      }
   }

   if(area->Flags & AREA_UNCONFIRMED)
   {
      sprintf(errorbuf,getString(ERR_UNKNOWN_AREA),areaname);
      LogWrite(1,TICKERR,errorbuf);
      BadTick(ticfilename,fullfilename,errorbuf);

      jbFreeList(&tick,sizeof(struct TickLine));
      return(TRUE);
   }

   strcpy(fullfilename,ticfilename);
   *PathPart(fullfilename)=0;
   AddPart(fullfilename,filename,200);

   for(tn=(struct TickNode *)area->TickNodes.First;tn;tn=tn->Next)
      if(tn->ConfigNode == cnode) break;

   if(!tn)
   {
      sprintf(errorbuf,getString(ERR_NODE_NOT_ATTACHED),
         cnode->Node.Zone,
         cnode->Node.Net,
         cnode->Node.Node,
         cnode->Node.Point,
         areaname);

      LogWrite(1,TICKERR,errorbuf);
      BadTick(ticfilename,fullfilename,errorbuf);

      jbFreeList(&tick,sizeof(struct TickLine));
      return(TRUE);
   }

   if(tn->Flags & TICKNODE_READONLY)
   {
      sprintf(errorbuf,getString(ERR_NODE_RECEIVE_ONLY),
         cnode->Node.Zone,
         cnode->Node.Net,
         cnode->Node.Node,
         cnode->Node.Point,
         areaname);

      LogWrite(1,TICKERR,errorbuf);
      BadTick(ticfilename,fullfilename,errorbuf);

      jbFreeList(&tick,sizeof(struct TickLine));
      return(TRUE);
   }

   if(crc!=0xffffffff && (config.cfg_Flags & CFG_CHECKCRC))
   {
      if(!CheckCRC(fullfilename,crc,errorbuf))
      {
         LogWrite(1,TICKERR,errorbuf);
         BadTick(ticfilename,fullfilename,errorbuf);

         jbFreeList(&tick,sizeof(struct TickLine));
         return(TRUE);
      }
   }

   res=WriteTick(&tick,area,fullfilename);

   jbFreeList(&tick,sizeof(struct TickLine));

   if(res)
      DeleteFile(ticfilename);

   return(res);
}

