/*

   NotifyErrors.rexx 2.0 by Johan Billing 1996

   This script extracts errors from your logfiles and post them in an
   echomail using CrashWrite. It's not fast, but it works! :-)

   This is an updated version of NotifyErrors.rexx that came in the archive
   of CrashMail 1.21. Changes:
   
   o Now you can use this ARexx script for scanning more than one logfile,
     for example both CrashTick.log and CrashMail.log!

   o You can configure the prefixes used to determine which lines should
     be extracted from the logfile and posted in an area.
     
   If you want to use another program than CrashWrite for generating the
   messages, it should be fairly easy to change the script a bit...

*/

/**** Configuration ****/

/* Full path and filename for CrashWrite */
crashwrite = "MAIL:CrashWrite"

/* Path to your inbound directory */
inbound = "MAIL:Inbound"

/* The originating node of the created messages */
node = "2:200/207.6"

/* The logfiles that should be scanned */

logfile.1  = "MAIL:CrashMail.log"      /* Name of the logfile      */
prefixes.1 = "!%"                      /* Prefixes to look for     */
area.1     = "KCC_ERRORS"              /* Where to post the result */

logfile.2  = "MAIL:CrashTick.log"
prefixes.2 = "!%"
area.2     = "KCC_ERRORS"

/* Important: Number of logfiles to scan */
logfiles = 2

/**** End of configuration ****/

options results

IF ~SHOW(Libraries,'rexxsupport.library') THEN
    IF ~ADDLIB("rexxsupport.library",0,-30,0) THEN EXIT

do i = 1 to logfiles
   say "Scanning " || logfile.i || "..."

   lines=0

   /* Start reading the log... */

   call open('file',logfile.i,'R')

   /* Skipping the part we have read already */

   pos=0

   do while ~eof('file')
      str=readln('file')

      if length(str)=47 then do
         if substr(str,21)="NotifyErrors.rexx logscan" then pos=seek('file',0,'Current')
      end
   end

   /* If we didn't find the string, the logfile must have been cleared and we
      should read it from the beginning */

   if substr(str,21)~="NotifyErrors.rexx logscan" then do
      call seek('file',pos,'B')
   end

   /* Scan for errors */

   do while ~eof('file')
      str = readln('file')

      if str~="" then do
         if pos(left(str,1),prefixes.i)~=0 then do

            if lines=0 then do
               /* Open the file if it is the first line */
               call open('outfile','T:NotifyErrors.tmp','W')
            end

            call writeln('outfile',str)
            lines=lines+1
         end
      end
   end

   call close('file')

   /* Write mark to logfile */

   call open('file',logfile.i,'A')
   rawdate=date('n')
   date=left(rawdate,2) || "-" || substr(rawdate,4,3) || "-" || substr(rawdate,10,2)

   /* "/" is the logfile prefix for miscellaneous information */

   call writeln('file',"/ " || date || " " || time('n') || "  " || "NotifyErrors.rexx logscan")
   close('file')

   /* If we found any lines... */

   if lines>0 then do
      /* Close tempfile */
      call writeln('outfile','')
      call close('outfile')

      /* *** WRITING *** */
      address command crashwrite 'FN "NotifyErrors" FA 'node' TN "All" TA 'node' SUBJ "Errors in ' || logfile.i || '" ORIGIN "Yet another interesting message" AREA "'area.i'" DIR "'inbound'" T:NotifyErrors.tmp'
      /* Yes, the line is VERY long... */

      call delete('T:NotifyErrors.tmp')
   end
end

