/*

   RaidUMS.rexx 1.0
   ================
   The purpose of this script is to generate messages in RFC-format from
   Raid requests that are present as mails in an UMS messagebase so that
   CrashTick can read and process them.

   This is how it works:
   ---------------------
   First you have to create a user with the name "Raid" or similar in
   your UMS messagebase. Then you run this script from time to time.
   RaidUMS.rexx checks if the user "Raid" has any unread mails. If it
   has, they are written to a temprary file in T: in RFC format and
   CrashTick is launched. CrashTick puts the responses in your inbound
   directory as Fidonet *.PKT files.

   Configuration:
   --------------
   umsusername
   umspassword
   umsserver

      RaidUMS.rexx checks for mails to 'umsusername' in the messagebase
      'umsserver' (normally set to 'default'). When logging in, the
      password 'umspassword' is used.

   crashtick

      Path and filename of 'CrashTick'

   ownadress.zone

      Here you configure with what address CrashTick should write its
      reponses to the processed requests. You have to specify an address
      for each zone that you expect to receive requests from. See the
      default configuration for an example.

*/

/* Configuration */

umsusername = "Raid"
umspassword = ""
umsserver = "default"
crashtick = "MAIL:CrashTick"

ownaddress.2 = "2:200/108.7"
ownaddress.39 = "39:162/504.24"

/* Script starts here */

signal on BREAK_C
signal on BREAK_D
signal on BREAK_E
signal on BREAK_F
signal on ERROR
signal on HALT
signal on IOERR
signal on SYNTAX

call addlib('ums.library', 0, -210, 11)
call addlib('rexxsupport.library', 0, -30, 1)

call UMSInitConsts()

options results

account = UMSLogin(umsusername,umspassword,umsserver)

if account=0 then do
 say 'Login failed'
 exit
end

emptyflags = UMSMakeFlags()
maskflags =  UMSMakeFlags(UMSUSTAT_ViewAccess, UMSUSTAT_Old)
matchflags = UMSMakeFlags(UMSUSTAT_ViewAccess)
flag0 = UMSMakeFlags(0)
flag1 = UMSMakeFlags(1)
flags01 = UMSMakeFlags(0,1)

/* set local flag 0 for all new messages */
call UMSSelectFlags(account, "L", flag0, emptyflags,,, "U", maskflags,matchflags)

/*  set local flag 1 on all mails */
call UMSSelectField(account, "L", flag1, emptyflags,,, UMSCODE_Group, "", TRUE)

/* main loop: read message, write message in RFC-format to T: and launch CrashTick */

last = 0

do forever
 /*  find next message with local flags 0 and 1 set */
 last = UMSSearchFlags(account, "LOCAL", flags01,flags01, last)
 if last=0 then leave

 drop msg.

 if UMSReadMsgAll(account, last, msg.,TRUE) then do
  if UMSReadMsgInfo(account, last, msg.) then do
   say "Found message from" msg.UMSCODE_FromName "at" msg.UMSCODE_FromAddr

   parse var msg.UMSCODE_FromAddr zone":"junk

   /* Check if an 'ownaddress' is set for this zone */
   if symbol('ownaddress.zone')~='VAR' then do
    say "Can't run CrashTick, no 'ownaddress' set for zone" zone
   end
   else do

    /* create message in RFC format */
    call open('file','T:RaidUMS.tmp','W')
    call writeln('file',"From: " || msg.UMSCODE_FromAddr || " (" || msg.UMSCODE_FromName || ")")
    call writeln('file',"To: " || ownaddress.zone || " (" || msg.UMSCODE_ToName || ")")
    call writeln('file',"Subject: " || msg.UMSCODE_Subject)
    call writeln('file',"")
    call writeln('file',msg.UMSCODE_MsgText)
    call close('file')

    /* Don't abort the script if CrashTick fails */
    signal off ERROR
    options failat 30

    /* Launch CrashTick */
    address command crashtick "RAID T:RaidUMS.tmp"
    err=rc

    /* Restore error checking */
    signal on ERROR
    options failat 10

    /* Only make the message Old if successful */
    if err=0 then do
     call UMSSelectMsg(account,"U", UMSMakeFlags(UMSUSTAT_Old), UMSMakeFlags(), last)
    end

    /* delete RFC message */
    call delete("T:RaidUMS.tmp")
   end
  end
  else call CheckErr
 end
 else call CheckErr
end

/*** Final cleanup ***/

BREAK_C:
BREAK_D:
BREAK_E:
BREAK_F:
HALT:

RC = 0

ERROR:
IOERR: 
SYNTAX:

IF RC ~= 0 THEN DO
    say "Error:" rc ERRORTEXT(rc) "in line" sigl
    END

/*** Logout ***/

IF account ~= 0 THEN DO
    CALL UMSLogout(account)
    account = 0
    END

EXIT

/*** Support ***/

CheckErr: PROCEDURE
    PARSE ARG account
    err = UMSErrNum(account)
    IF err ~= 0 THEN DO
        SAY "UMS Error #" || err || ": " || UMSErrTxt(account)
    END
    RETURN
